from bingads.v13.bulk.entities import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *
from .bulk_conversion_goal import BulkConversionGoal
from decimal import Decimal

class BulkEventGoal(BulkConversionGoal):
    """ Represents an Event Goal that can be read or written in a bulk file.

    This class exposes the :attr:`event_goal` property that can be read and written as fields of the
    Event Goal record in a bulk file.

    For more information, see Event Goal at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 event_goal=None):
        super(BulkEventGoal, self).__init__(conversion_goal = event_goal)

    _MAPPINGS = [

        _SimpleBulkMapping(
            header=_StringTable.CategoryExpression,
            field_to_csv=lambda c: bulk_str(c.event_goal.CategoryExpression),
            csv_to_field=lambda c, v: setattr(c.event_goal, 'CategoryExpression', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CategoryOperator,
            field_to_csv=lambda c: bulk_str(c.event_goal.CategoryOperator),
            csv_to_field=lambda c, v: setattr(c.event_goal, 'CategoryOperator', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ActionExpression,
            field_to_csv=lambda c: bulk_str(c.event_goal.ActionExpression),
            csv_to_field=lambda c, v: setattr(c.event_goal, 'ActionExpression', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ActionOperator,
            field_to_csv=lambda c: bulk_str(c.event_goal.ActionOperator),
            csv_to_field=lambda c, v: setattr(c.event_goal, 'ActionOperator', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.LabelExpression,
            field_to_csv=lambda c: bulk_str(c.event_goal.LabelExpression),
            csv_to_field=lambda c, v: setattr(c.event_goal, 'LabelExpression', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.LabelOperator,
            field_to_csv=lambda c: bulk_str(c.event_goal.LabelOperator),
            csv_to_field=lambda c, v: setattr(c.event_goal, 'LabelOperator', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EventValue,
            field_to_csv=lambda c: bulk_str(c.event_goal.Value),
            csv_to_field=lambda c, v: setattr(c.event_goal, 'Value', Decimal(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EventValueOperator,
            field_to_csv=lambda c: bulk_str(c.event_goal.ValueOperator),
            csv_to_field=lambda c, v: setattr(c.event_goal, 'ValueOperator', v)
        ),
    ]

    @property
    def event_goal(self):
        """ Defines a Event Goal """

        return self._conversion_goal

    @event_goal.setter
    def event_goal(self, event_goal):
        self._conversion_goal = event_goal


    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.event_goal, 'event_goal')
        super(BulkEventGoal, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkEventGoal._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self.event_goal = _CAMPAIGN_OBJECT_FACTORY_V13.create('EventGoal')
        super(BulkEventGoal, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkEventGoal._MAPPINGS)




# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class PerformanceInsightsUrlId(str, Enum):
    RECRECOMMENDATIONPAGE = 'RecRecommendationPage'

    RECCOMPETITIONPAGE = 'RecCompetitionPage'

    SETTINGCREATEORDER = 'SettingCreateOrder'

    SETTINGREMOVEHOLD = 'SettingRemoveHold'

    SETTINGADDFUNDS = 'SettingAddFunds'

    REPORTSEARCHTERM = 'ReportSearchTerm'

    REPORTPUBLISHERWEBSITE = 'ReportPublisherWebsite'

    REPORTAUCTIONINSIGHTS = 'ReportAuctionInsights'

    HELPDOCNETWORKSETTING = 'HelpDocNetworkSetting'

    HELPDOCEDITORIAL = 'HelpDocEditorial'

    HELPDOCWEBSITEREPORT = 'HelpDocWebsiteReport'

    HELPDOCWEBSITEEXCLUSION = 'HelpDocWebsiteExclusion'

    HELPDOCCREATEORDER = 'HelpDocCreateOrder'

    HELPDOCREMOVEHOLD = 'HelpDocRemoveHold'

    HELPDOCADDFUNDS = 'HelpDocAddFunds'

    HELPDOCRECOMMENDATION = 'HelpDocRecommendation'

    HELPDOCAUDIENCENETWORK = 'HelpDocAudienceNetwork'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PerformanceInsightsUrlId from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.asset_group_editorial_status import AssetGroupEditorialStatus
from openapi_client.models.campaign.asset_group_search_theme import AssetGroupSearchTheme
from openapi_client.models.campaign.asset_group_status import AssetGroupStatus
from openapi_client.models.campaign.asset_group_url_target import AssetGroupUrlTarget
from openapi_client.models.campaign.asset_link import AssetLink
from openapi_client.models.campaign.call_to_action import CallToAction
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.campaign.model_date import ModelDate
from typing_extensions import Self

class AssetGroup(BaseModel):
    """
    AssetGroup
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    status: Optional[AssetGroupStatus] = Field(default=None, alias="Status")
    start_date: Optional[ModelDate] = Field(default=None, alias="StartDate")
    end_date: Optional[ModelDate] = Field(default=None, alias="EndDate")
    path1: Optional[StrictStr] = Field(default=None, alias="Path1")
    path2: Optional[StrictStr] = Field(default=None, alias="Path2")
    business_name: Optional[StrictStr] = Field(default=None, alias="BusinessName")
    headlines: Optional[List[Optional[AssetLink]]] = Field(default=None, alias="Headlines")
    long_headlines: Optional[List[Optional[AssetLink]]] = Field(default=None, alias="LongHeadlines")
    descriptions: Optional[List[Optional[AssetLink]]] = Field(default=None, alias="Descriptions")
    images: Optional[List[Optional[AssetLink]]] = Field(default=None, alias="Images")
    videos: Optional[List[Optional[AssetLink]]] = Field(default=None, alias="Videos")
    call_to_action: Optional[CallToAction] = Field(default=None, alias="CallToAction")
    final_urls: Optional[List[StrictStr]] = Field(default=None, alias="FinalUrls")
    asset_group_search_themes: Optional[List[Optional[AssetGroupSearchTheme]]] = Field(default=None, alias="AssetGroupSearchThemes")
    final_mobile_urls: Optional[List[StrictStr]] = Field(default=None, alias="FinalMobileUrls")
    editorial_status: Optional[AssetGroupEditorialStatus] = Field(default=None, alias="EditorialStatus")
    asset_group_url_targets: Optional[List[Optional[AssetGroupUrlTarget]]] = Field(default=None, alias="AssetGroupUrlTargets")
    forward_compatibility_map: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="ForwardCompatibilityMap")
    __properties: ClassVar[List[str]] = ["Id", "Name", "Status", "StartDate", "EndDate", "Path1", "Path2", "BusinessName", "Headlines", "LongHeadlines", "Descriptions", "Images", "Videos", "CallToAction", "FinalUrls", "AssetGroupSearchThemes", "FinalMobileUrls", "EditorialStatus", "AssetGroupUrlTargets", "ForwardCompatibilityMap"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of start_date
        if self.start_date:
            _dict['StartDate'] = self.start_date.to_dict()
        # override the default output from pydantic by calling `to_dict()` of end_date
        if self.end_date:
            _dict['EndDate'] = self.end_date.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in headlines (list)
        _items = []
        if self.headlines:
            for _item_headlines in self.headlines:
                if _item_headlines:
                    _items.append(_item_headlines.to_dict())
            _dict['Headlines'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in long_headlines (list)
        _items = []
        if self.long_headlines:
            for _item_long_headlines in self.long_headlines:
                if _item_long_headlines:
                    _items.append(_item_long_headlines.to_dict())
            _dict['LongHeadlines'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in descriptions (list)
        _items = []
        if self.descriptions:
            for _item_descriptions in self.descriptions:
                if _item_descriptions:
                    _items.append(_item_descriptions.to_dict())
            _dict['Descriptions'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in images (list)
        _items = []
        if self.images:
            for _item_images in self.images:
                if _item_images:
                    _items.append(_item_images.to_dict())
            _dict['Images'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in videos (list)
        _items = []
        if self.videos:
            for _item_videos in self.videos:
                if _item_videos:
                    _items.append(_item_videos.to_dict())
            _dict['Videos'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in asset_group_search_themes (list)
        _items = []
        if self.asset_group_search_themes:
            for _item_asset_group_search_themes in self.asset_group_search_themes:
                if _item_asset_group_search_themes:
                    _items.append(_item_asset_group_search_themes.to_dict())
            _dict['AssetGroupSearchThemes'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in asset_group_url_targets (list)
        _items = []
        if self.asset_group_url_targets:
            for _item_asset_group_url_targets in self.asset_group_url_targets:
                if _item_asset_group_url_targets:
                    _items.append(_item_asset_group_url_targets.to_dict())
            _dict['AssetGroupUrlTargets'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in forward_compatibility_map (list)
        _items = []
        if self.forward_compatibility_map:
            for _item_forward_compatibility_map in self.forward_compatibility_map:
                if _item_forward_compatibility_map:
                    _items.append(_item_forward_compatibility_map.to_dict())
            _dict['ForwardCompatibilityMap'] = _items
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        # set to None if start_date (nullable) is None
        # and model_fields_set contains the field
        if self.start_date is None and "start_date" in self.model_fields_set:
            _dict['StartDate'] = None

        # set to None if end_date (nullable) is None
        # and model_fields_set contains the field
        if self.end_date is None and "end_date" in self.model_fields_set:
            _dict['EndDate'] = None

        # set to None if path1 (nullable) is None
        # and model_fields_set contains the field
        if self.path1 is None and "path1" in self.model_fields_set:
            _dict['Path1'] = None

        # set to None if path2 (nullable) is None
        # and model_fields_set contains the field
        if self.path2 is None and "path2" in self.model_fields_set:
            _dict['Path2'] = None

        # set to None if business_name (nullable) is None
        # and model_fields_set contains the field
        if self.business_name is None and "business_name" in self.model_fields_set:
            _dict['BusinessName'] = None

        # set to None if headlines (nullable) is None
        # and model_fields_set contains the field
        if self.headlines is None and "headlines" in self.model_fields_set:
            _dict['Headlines'] = None

        # set to None if long_headlines (nullable) is None
        # and model_fields_set contains the field
        if self.long_headlines is None and "long_headlines" in self.model_fields_set:
            _dict['LongHeadlines'] = None

        # set to None if descriptions (nullable) is None
        # and model_fields_set contains the field
        if self.descriptions is None and "descriptions" in self.model_fields_set:
            _dict['Descriptions'] = None

        # set to None if images (nullable) is None
        # and model_fields_set contains the field
        if self.images is None and "images" in self.model_fields_set:
            _dict['Images'] = None

        # set to None if videos (nullable) is None
        # and model_fields_set contains the field
        if self.videos is None and "videos" in self.model_fields_set:
            _dict['Videos'] = None

        # set to None if call_to_action (nullable) is None
        # and model_fields_set contains the field
        if self.call_to_action is None and "call_to_action" in self.model_fields_set:
            _dict['CallToAction'] = None

        # set to None if final_urls (nullable) is None
        # and model_fields_set contains the field
        if self.final_urls is None and "final_urls" in self.model_fields_set:
            _dict['FinalUrls'] = None

        # set to None if asset_group_search_themes (nullable) is None
        # and model_fields_set contains the field
        if self.asset_group_search_themes is None and "asset_group_search_themes" in self.model_fields_set:
            _dict['AssetGroupSearchThemes'] = None

        # set to None if final_mobile_urls (nullable) is None
        # and model_fields_set contains the field
        if self.final_mobile_urls is None and "final_mobile_urls" in self.model_fields_set:
            _dict['FinalMobileUrls'] = None

        # set to None if editorial_status (nullable) is None
        # and model_fields_set contains the field
        if self.editorial_status is None and "editorial_status" in self.model_fields_set:
            _dict['EditorialStatus'] = None

        # set to None if asset_group_url_targets (nullable) is None
        # and model_fields_set contains the field
        if self.asset_group_url_targets is None and "asset_group_url_targets" in self.model_fields_set:
            _dict['AssetGroupUrlTargets'] = None

        # set to None if forward_compatibility_map (nullable) is None
        # and model_fields_set contains the field
        if self.forward_compatibility_map is None and "forward_compatibility_map" in self.model_fields_set:
            _dict['ForwardCompatibilityMap'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AssetGroup from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None,
                        "StartDate": ModelDate.from_dict(obj["StartDate"]) if obj.get("StartDate") is not None else None,
                        "EndDate": ModelDate.from_dict(obj["EndDate"]) if obj.get("EndDate") is not None else None,
                        "Path1": obj.get("Path1") if obj.get("Path1") is not None else None,
                        "Path2": obj.get("Path2") if obj.get("Path2") is not None else None,
                        "BusinessName": obj.get("BusinessName") if obj.get("BusinessName") is not None else None,
                        "Headlines": [AssetLink.from_dict(_item) for _item in obj["Headlines"]] if obj.get("Headlines") is not None else None,
                        "LongHeadlines": [AssetLink.from_dict(_item) for _item in obj["LongHeadlines"]] if obj.get("LongHeadlines") is not None else None,
                        "Descriptions": [AssetLink.from_dict(_item) for _item in obj["Descriptions"]] if obj.get("Descriptions") is not None else None,
                        "Images": [AssetLink.from_dict(_item) for _item in obj["Images"]] if obj.get("Images") is not None else None,
                        "Videos": [AssetLink.from_dict(_item) for _item in obj["Videos"]] if obj.get("Videos") is not None else None,
                        "CallToAction": obj.get("CallToAction") if obj.get("CallToAction") is not None else None,
                        "FinalUrls": obj.get("FinalUrls"),
                        "AssetGroupSearchThemes": [AssetGroupSearchTheme.from_dict(_item) for _item in obj["AssetGroupSearchThemes"]] if obj.get("AssetGroupSearchThemes") is not None else None,
                        "FinalMobileUrls": obj.get("FinalMobileUrls"),
                        "EditorialStatus": obj.get("EditorialStatus") if obj.get("EditorialStatus") is not None else None,
                        "AssetGroupUrlTargets": [AssetGroupUrlTarget.from_dict(_item) for _item in obj["AssetGroupUrlTargets"]] if obj.get("AssetGroupUrlTargets") is not None else None,
                        "ForwardCompatibilityMap": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["ForwardCompatibilityMap"]] if obj.get("ForwardCompatibilityMap") is not None else None
        })
        return _obj



from bingads.v13.bulk.entities import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.bulk.entities.bulk_ad_group_criterion import BulkAdGroupCriterion
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable

_HotelGroup = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('HotelGroup'))

class BulkAdGroupHotelListingGroup(BulkAdGroupCriterion):
    """ Represents an Ad Group Hotel Group that can be read or written in a bulk file.

    This class exposes the :attr:`ad_group_criterion` property that can be read and written as fields of the
    Ad Group Hotel Group record in a bulk file.

    For more information, see Ad Group Product Scope at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 campaign_name=None,
                 ad_group_name=None,
                 ad_group_criterion=None
                 ):
        super(BulkAdGroupHotelListingGroup, self).__init__(campaign_name, ad_group_name, ad_group_criterion)

    @classmethod
    def _get_listing_type(cls, entity):
        if entity.ad_group_criterion.Criterion is not None and \
            hasattr(entity.ad_group_criterion.Criterion, 'ListingType'):
            return entity.ad_group_criterion.Criterion.ListingType
        return None

    @classmethod
    def _get_parent_criterion_id(cls, entity):
        if entity.ad_group_criterion.Criterion is not None and \
            hasattr(entity.ad_group_criterion.Criterion, 'ParentCriterionId'):
            return bulk_str(entity.ad_group_criterion.Criterion.ParentCriterionId)
        return None

    @classmethod
    def _get_listing_operand(cls, entity):
        if entity.ad_group_criterion.Criterion is not None and \
            hasattr(entity.ad_group_criterion.Criterion, 'Listing') and \
            entity.ad_group_criterion.Criterion.Listing is not None and \
            hasattr(entity.ad_group_criterion.Criterion.Listing, 'Operand'):
            return entity.ad_group_criterion.Criterion.Listing.Operand
        return None

    @classmethod
    def _get_listing_attribute(cls, entity):
        if entity.ad_group_criterion.Criterion is not None and \
            hasattr(entity.ad_group_criterion.Criterion, 'Listing') and \
            entity.ad_group_criterion.Criterion.Listing is not None and \
            hasattr(entity.ad_group_criterion.Criterion.Listing, 'Attribute'):
            return entity.ad_group_criterion.Criterion.Listing.Attribute
        return None

    _MAPPINGS = [
        _SimpleBulkMapping(
            _StringTable.SubType,
            field_to_csv=lambda c: BulkAdGroupHotelListingGroup._get_listing_type(c),
            csv_to_field=lambda c, v: setattr(c.ad_group_criterion.Criterion, 'ListingType', v)
        ),
        _SimpleBulkMapping(
            _StringTable.ParentAdGroupCriterionId,
            field_to_csv=lambda c: BulkAdGroupHotelListingGroup._get_parent_criterion_id(c),
            csv_to_field=lambda c, v: setattr(c.ad_group_criterion.Criterion, 'ParentCriterionId',
                                              int(v) if v else None)
        ),
        _SimpleBulkMapping(
            _StringTable.HotelAttribute,
            field_to_csv=lambda c: BulkAdGroupHotelListingGroup._get_listing_operand(c),
            csv_to_field=lambda c, v: setattr(c.ad_group_criterion.Criterion.Listing, 'Operand', v)
        ),
        _SimpleBulkMapping(
            _StringTable.HotelAttributeValue,
            field_to_csv=lambda c: BulkAdGroupHotelListingGroup._get_listing_attribute(c),
            csv_to_field=lambda c, v: setattr(c.ad_group_criterion.Criterion.Listing, 'Attribute', v)
        ),
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        super(BulkAdGroupHotelListingGroup, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkAdGroupHotelListingGroup._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        super(BulkAdGroupHotelListingGroup, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkAdGroupHotelListingGroup._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAdGroupHotelListingGroup, self).read_additional_data(stream_reader)

    def create_criterion(self):
        hotel_group = _CAMPAIGN_OBJECT_FACTORY_V13.create('HotelGroup')
        hotel_group.Listing = _CAMPAIGN_OBJECT_FACTORY_V13.create('HotelListing')
        hotel_group.Type = 'HotelGroup'
        return hotel_group



# coding: utf-8

"""
    CustomerBillingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from openapi_client.models.billing.add_insertion_order_request import AddInsertionOrderRequest
from openapi_client.models.billing.add_insertion_order_response import AddInsertionOrderResponse
from openapi_client.models.billing.check_feature_adoption_coupon_eligibility_request import CheckFeatureAdoptionCouponEligibilityRequest
from openapi_client.models.billing.check_feature_adoption_coupon_eligibility_response import CheckFeatureAdoptionCouponEligibilityResponse
from openapi_client.models.billing.claim_feature_adoption_coupons_request import ClaimFeatureAdoptionCouponsRequest
from openapi_client.models.billing.claim_feature_adoption_coupons_response import ClaimFeatureAdoptionCouponsResponse
from openapi_client.models.billing.dispatch_coupons_request import DispatchCouponsRequest
from openapi_client.models.billing.dispatch_coupons_response import DispatchCouponsResponse
from openapi_client.models.billing.get_account_monthly_spend_request import GetAccountMonthlySpendRequest
from openapi_client.models.billing.get_account_monthly_spend_response import GetAccountMonthlySpendResponse
from openapi_client.models.billing.get_billing_documents_info_request import GetBillingDocumentsInfoRequest
from openapi_client.models.billing.get_billing_documents_info_response import GetBillingDocumentsInfoResponse
from openapi_client.models.billing.get_billing_documents_request import GetBillingDocumentsRequest
from openapi_client.models.billing.get_billing_documents_response import GetBillingDocumentsResponse
from openapi_client.models.billing.get_coupon_info_request import GetCouponInfoRequest
from openapi_client.models.billing.get_coupon_info_response import GetCouponInfoResponse
from openapi_client.models.billing.redeem_coupon_request import RedeemCouponRequest
from openapi_client.models.billing.redeem_coupon_response import RedeemCouponResponse
from openapi_client.models.billing.search_coupons_request import SearchCouponsRequest
from openapi_client.models.billing.search_coupons_response import SearchCouponsResponse
from openapi_client.models.billing.search_insertion_orders_request import SearchInsertionOrdersRequest
from openapi_client.models.billing.search_insertion_orders_response import SearchInsertionOrdersResponse
from openapi_client.models.billing.update_insertion_order_request import UpdateInsertionOrderRequest
from openapi_client.models.billing.update_insertion_order_response import UpdateInsertionOrderResponse

from openapi_client.api_client import ApiClient, RequestSerialized
from openapi_client.api_response import ApiResponse
from openapi_client.rest import RESTResponseType


class CustomerBillingServiceApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def add_insertion_order(
        self,
        add_insertion_order_request: AddInsertionOrderRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AddInsertionOrderResponse:
        """add_insertion_order


        :param add_insertion_order_request: (required)
        :type add_insertion_order_request: AddInsertionOrderRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._add_insertion_order_serialize(
            add_insertion_order_request=add_insertion_order_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AddInsertionOrderResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def add_insertion_order_with_http_info(
        self,
        add_insertion_order_request: AddInsertionOrderRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AddInsertionOrderResponse]:
        """add_insertion_order


        :param add_insertion_order_request: (required)
        :type add_insertion_order_request: AddInsertionOrderRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._add_insertion_order_serialize(
            add_insertion_order_request=add_insertion_order_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AddInsertionOrderResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def add_insertion_order_without_preload_content(
        self,
        add_insertion_order_request: AddInsertionOrderRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """add_insertion_order


        :param add_insertion_order_request: (required)
        :type add_insertion_order_request: AddInsertionOrderRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._add_insertion_order_serialize(
            add_insertion_order_request=add_insertion_order_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AddInsertionOrderResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _add_insertion_order_serialize(
        self,
        add_insertion_order_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if add_insertion_order_request is not None:
            _body_params = add_insertion_order_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/InsertionOrder',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def check_feature_adoption_coupon_eligibility(
        self,
        check_feature_adoption_coupon_eligibility_request: CheckFeatureAdoptionCouponEligibilityRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CheckFeatureAdoptionCouponEligibilityResponse:
        """check_feature_adoption_coupon_eligibility


        :param check_feature_adoption_coupon_eligibility_request: (required)
        :type check_feature_adoption_coupon_eligibility_request: CheckFeatureAdoptionCouponEligibilityRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._check_feature_adoption_coupon_eligibility_serialize(
            check_feature_adoption_coupon_eligibility_request=check_feature_adoption_coupon_eligibility_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CheckFeatureAdoptionCouponEligibilityResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def check_feature_adoption_coupon_eligibility_with_http_info(
        self,
        check_feature_adoption_coupon_eligibility_request: CheckFeatureAdoptionCouponEligibilityRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CheckFeatureAdoptionCouponEligibilityResponse]:
        """check_feature_adoption_coupon_eligibility


        :param check_feature_adoption_coupon_eligibility_request: (required)
        :type check_feature_adoption_coupon_eligibility_request: CheckFeatureAdoptionCouponEligibilityRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._check_feature_adoption_coupon_eligibility_serialize(
            check_feature_adoption_coupon_eligibility_request=check_feature_adoption_coupon_eligibility_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CheckFeatureAdoptionCouponEligibilityResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def check_feature_adoption_coupon_eligibility_without_preload_content(
        self,
        check_feature_adoption_coupon_eligibility_request: CheckFeatureAdoptionCouponEligibilityRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """check_feature_adoption_coupon_eligibility


        :param check_feature_adoption_coupon_eligibility_request: (required)
        :type check_feature_adoption_coupon_eligibility_request: CheckFeatureAdoptionCouponEligibilityRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._check_feature_adoption_coupon_eligibility_serialize(
            check_feature_adoption_coupon_eligibility_request=check_feature_adoption_coupon_eligibility_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CheckFeatureAdoptionCouponEligibilityResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _check_feature_adoption_coupon_eligibility_serialize(
        self,
        check_feature_adoption_coupon_eligibility_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if check_feature_adoption_coupon_eligibility_request is not None:
            _body_params = check_feature_adoption_coupon_eligibility_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/FeatureAdoptionCouponEligibility/Check',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def claim_feature_adoption_coupons(
        self,
        claim_feature_adoption_coupons_request: ClaimFeatureAdoptionCouponsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ClaimFeatureAdoptionCouponsResponse:
        """claim_feature_adoption_coupons


        :param claim_feature_adoption_coupons_request: (required)
        :type claim_feature_adoption_coupons_request: ClaimFeatureAdoptionCouponsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._claim_feature_adoption_coupons_serialize(
            claim_feature_adoption_coupons_request=claim_feature_adoption_coupons_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ClaimFeatureAdoptionCouponsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def claim_feature_adoption_coupons_with_http_info(
        self,
        claim_feature_adoption_coupons_request: ClaimFeatureAdoptionCouponsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ClaimFeatureAdoptionCouponsResponse]:
        """claim_feature_adoption_coupons


        :param claim_feature_adoption_coupons_request: (required)
        :type claim_feature_adoption_coupons_request: ClaimFeatureAdoptionCouponsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._claim_feature_adoption_coupons_serialize(
            claim_feature_adoption_coupons_request=claim_feature_adoption_coupons_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ClaimFeatureAdoptionCouponsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def claim_feature_adoption_coupons_without_preload_content(
        self,
        claim_feature_adoption_coupons_request: ClaimFeatureAdoptionCouponsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """claim_feature_adoption_coupons


        :param claim_feature_adoption_coupons_request: (required)
        :type claim_feature_adoption_coupons_request: ClaimFeatureAdoptionCouponsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._claim_feature_adoption_coupons_serialize(
            claim_feature_adoption_coupons_request=claim_feature_adoption_coupons_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ClaimFeatureAdoptionCouponsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _claim_feature_adoption_coupons_serialize(
        self,
        claim_feature_adoption_coupons_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if claim_feature_adoption_coupons_request is not None:
            _body_params = claim_feature_adoption_coupons_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/FeatureAdoptionCoupons/Claim',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def dispatch_coupons(
        self,
        dispatch_coupons_request: DispatchCouponsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> DispatchCouponsResponse:
        """dispatch_coupons


        :param dispatch_coupons_request: (required)
        :type dispatch_coupons_request: DispatchCouponsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._dispatch_coupons_serialize(
            dispatch_coupons_request=dispatch_coupons_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "DispatchCouponsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def dispatch_coupons_with_http_info(
        self,
        dispatch_coupons_request: DispatchCouponsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[DispatchCouponsResponse]:
        """dispatch_coupons


        :param dispatch_coupons_request: (required)
        :type dispatch_coupons_request: DispatchCouponsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._dispatch_coupons_serialize(
            dispatch_coupons_request=dispatch_coupons_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "DispatchCouponsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def dispatch_coupons_without_preload_content(
        self,
        dispatch_coupons_request: DispatchCouponsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """dispatch_coupons


        :param dispatch_coupons_request: (required)
        :type dispatch_coupons_request: DispatchCouponsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._dispatch_coupons_serialize(
            dispatch_coupons_request=dispatch_coupons_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "DispatchCouponsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _dispatch_coupons_serialize(
        self,
        dispatch_coupons_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if dispatch_coupons_request is not None:
            _body_params = dispatch_coupons_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/Coupons/Dispatch',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_account_monthly_spend(
        self,
        get_account_monthly_spend_request: GetAccountMonthlySpendRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetAccountMonthlySpendResponse:
        """get_account_monthly_spend


        :param get_account_monthly_spend_request: (required)
        :type get_account_monthly_spend_request: GetAccountMonthlySpendRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_account_monthly_spend_serialize(
            get_account_monthly_spend_request=get_account_monthly_spend_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetAccountMonthlySpendResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_account_monthly_spend_with_http_info(
        self,
        get_account_monthly_spend_request: GetAccountMonthlySpendRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetAccountMonthlySpendResponse]:
        """get_account_monthly_spend


        :param get_account_monthly_spend_request: (required)
        :type get_account_monthly_spend_request: GetAccountMonthlySpendRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_account_monthly_spend_serialize(
            get_account_monthly_spend_request=get_account_monthly_spend_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetAccountMonthlySpendResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_account_monthly_spend_without_preload_content(
        self,
        get_account_monthly_spend_request: GetAccountMonthlySpendRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """get_account_monthly_spend


        :param get_account_monthly_spend_request: (required)
        :type get_account_monthly_spend_request: GetAccountMonthlySpendRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_account_monthly_spend_serialize(
            get_account_monthly_spend_request=get_account_monthly_spend_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetAccountMonthlySpendResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_account_monthly_spend_serialize(
        self,
        get_account_monthly_spend_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_account_monthly_spend_request is not None:
            _body_params = get_account_monthly_spend_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/AccountMonthlySpend/Query',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_billing_documents(
        self,
        get_billing_documents_request: GetBillingDocumentsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetBillingDocumentsResponse:
        """get_billing_documents


        :param get_billing_documents_request: (required)
        :type get_billing_documents_request: GetBillingDocumentsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_billing_documents_serialize(
            get_billing_documents_request=get_billing_documents_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetBillingDocumentsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_billing_documents_with_http_info(
        self,
        get_billing_documents_request: GetBillingDocumentsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetBillingDocumentsResponse]:
        """get_billing_documents


        :param get_billing_documents_request: (required)
        :type get_billing_documents_request: GetBillingDocumentsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_billing_documents_serialize(
            get_billing_documents_request=get_billing_documents_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetBillingDocumentsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_billing_documents_without_preload_content(
        self,
        get_billing_documents_request: GetBillingDocumentsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """get_billing_documents


        :param get_billing_documents_request: (required)
        :type get_billing_documents_request: GetBillingDocumentsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_billing_documents_serialize(
            get_billing_documents_request=get_billing_documents_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetBillingDocumentsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_billing_documents_serialize(
        self,
        get_billing_documents_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_billing_documents_request is not None:
            _body_params = get_billing_documents_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/BillingDocuments/Query',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_billing_documents_info(
        self,
        get_billing_documents_info_request: GetBillingDocumentsInfoRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetBillingDocumentsInfoResponse:
        """get_billing_documents_info


        :param get_billing_documents_info_request: (required)
        :type get_billing_documents_info_request: GetBillingDocumentsInfoRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_billing_documents_info_serialize(
            get_billing_documents_info_request=get_billing_documents_info_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetBillingDocumentsInfoResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_billing_documents_info_with_http_info(
        self,
        get_billing_documents_info_request: GetBillingDocumentsInfoRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetBillingDocumentsInfoResponse]:
        """get_billing_documents_info


        :param get_billing_documents_info_request: (required)
        :type get_billing_documents_info_request: GetBillingDocumentsInfoRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_billing_documents_info_serialize(
            get_billing_documents_info_request=get_billing_documents_info_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetBillingDocumentsInfoResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_billing_documents_info_without_preload_content(
        self,
        get_billing_documents_info_request: GetBillingDocumentsInfoRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """get_billing_documents_info


        :param get_billing_documents_info_request: (required)
        :type get_billing_documents_info_request: GetBillingDocumentsInfoRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_billing_documents_info_serialize(
            get_billing_documents_info_request=get_billing_documents_info_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetBillingDocumentsInfoResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_billing_documents_info_serialize(
        self,
        get_billing_documents_info_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_billing_documents_info_request is not None:
            _body_params = get_billing_documents_info_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/BillingDocumentsInfo/Query',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_coupon_info(
        self,
        get_coupon_info_request: GetCouponInfoRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCouponInfoResponse:
        """get_coupon_info


        :param get_coupon_info_request: (required)
        :type get_coupon_info_request: GetCouponInfoRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_coupon_info_serialize(
            get_coupon_info_request=get_coupon_info_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCouponInfoResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_coupon_info_with_http_info(
        self,
        get_coupon_info_request: GetCouponInfoRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCouponInfoResponse]:
        """get_coupon_info


        :param get_coupon_info_request: (required)
        :type get_coupon_info_request: GetCouponInfoRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_coupon_info_serialize(
            get_coupon_info_request=get_coupon_info_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCouponInfoResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_coupon_info_without_preload_content(
        self,
        get_coupon_info_request: GetCouponInfoRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """get_coupon_info


        :param get_coupon_info_request: (required)
        :type get_coupon_info_request: GetCouponInfoRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_coupon_info_serialize(
            get_coupon_info_request=get_coupon_info_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCouponInfoResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_coupon_info_serialize(
        self,
        get_coupon_info_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_coupon_info_request is not None:
            _body_params = get_coupon_info_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/CouponInfo/Query',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def redeem_coupon(
        self,
        redeem_coupon_request: RedeemCouponRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RedeemCouponResponse:
        """redeem_coupon


        :param redeem_coupon_request: (required)
        :type redeem_coupon_request: RedeemCouponRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._redeem_coupon_serialize(
            redeem_coupon_request=redeem_coupon_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RedeemCouponResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def redeem_coupon_with_http_info(
        self,
        redeem_coupon_request: RedeemCouponRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[RedeemCouponResponse]:
        """redeem_coupon


        :param redeem_coupon_request: (required)
        :type redeem_coupon_request: RedeemCouponRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._redeem_coupon_serialize(
            redeem_coupon_request=redeem_coupon_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RedeemCouponResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def redeem_coupon_without_preload_content(
        self,
        redeem_coupon_request: RedeemCouponRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """redeem_coupon


        :param redeem_coupon_request: (required)
        :type redeem_coupon_request: RedeemCouponRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._redeem_coupon_serialize(
            redeem_coupon_request=redeem_coupon_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RedeemCouponResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _redeem_coupon_serialize(
        self,
        redeem_coupon_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if redeem_coupon_request is not None:
            _body_params = redeem_coupon_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/Coupon/Redeem',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def search_coupons(
        self,
        search_coupons_request: SearchCouponsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SearchCouponsResponse:
        """search_coupons


        :param search_coupons_request: (required)
        :type search_coupons_request: SearchCouponsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._search_coupons_serialize(
            search_coupons_request=search_coupons_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchCouponsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def search_coupons_with_http_info(
        self,
        search_coupons_request: SearchCouponsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SearchCouponsResponse]:
        """search_coupons


        :param search_coupons_request: (required)
        :type search_coupons_request: SearchCouponsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._search_coupons_serialize(
            search_coupons_request=search_coupons_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchCouponsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def search_coupons_without_preload_content(
        self,
        search_coupons_request: SearchCouponsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """search_coupons


        :param search_coupons_request: (required)
        :type search_coupons_request: SearchCouponsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._search_coupons_serialize(
            search_coupons_request=search_coupons_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchCouponsResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _search_coupons_serialize(
        self,
        search_coupons_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if search_coupons_request is not None:
            _body_params = search_coupons_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/Coupons/Search',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def search_insertion_orders(
        self,
        search_insertion_orders_request: SearchInsertionOrdersRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SearchInsertionOrdersResponse:
        """search_insertion_orders


        :param search_insertion_orders_request: (required)
        :type search_insertion_orders_request: SearchInsertionOrdersRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._search_insertion_orders_serialize(
            search_insertion_orders_request=search_insertion_orders_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchInsertionOrdersResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def search_insertion_orders_with_http_info(
        self,
        search_insertion_orders_request: SearchInsertionOrdersRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SearchInsertionOrdersResponse]:
        """search_insertion_orders


        :param search_insertion_orders_request: (required)
        :type search_insertion_orders_request: SearchInsertionOrdersRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._search_insertion_orders_serialize(
            search_insertion_orders_request=search_insertion_orders_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchInsertionOrdersResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def search_insertion_orders_without_preload_content(
        self,
        search_insertion_orders_request: SearchInsertionOrdersRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """search_insertion_orders


        :param search_insertion_orders_request: (required)
        :type search_insertion_orders_request: SearchInsertionOrdersRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._search_insertion_orders_serialize(
            search_insertion_orders_request=search_insertion_orders_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchInsertionOrdersResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _search_insertion_orders_serialize(
        self,
        search_insertion_orders_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if search_insertion_orders_request is not None:
            _body_params = search_insertion_orders_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/CustomerBilling/v13/InsertionOrders/Search',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def update_insertion_order(
        self,
        update_insertion_order_request: UpdateInsertionOrderRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UpdateInsertionOrderResponse:
        """update_insertion_order


        :param update_insertion_order_request: (required)
        :type update_insertion_order_request: UpdateInsertionOrderRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_insertion_order_serialize(
            update_insertion_order_request=update_insertion_order_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UpdateInsertionOrderResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_insertion_order_with_http_info(
        self,
        update_insertion_order_request: UpdateInsertionOrderRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UpdateInsertionOrderResponse]:
        """update_insertion_order


        :param update_insertion_order_request: (required)
        :type update_insertion_order_request: UpdateInsertionOrderRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_insertion_order_serialize(
            update_insertion_order_request=update_insertion_order_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UpdateInsertionOrderResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def update_insertion_order_without_preload_content(
        self,
        update_insertion_order_request: UpdateInsertionOrderRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """update_insertion_order


        :param update_insertion_order_request: (required)
        :type update_insertion_order_request: UpdateInsertionOrderRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_insertion_order_serialize(
            update_insertion_order_request=update_insertion_order_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UpdateInsertionOrderResponse",
            '400': "ApplicationFault",
            '500': "ApplicationFault",
            '401': "ApplicationFault",
            '403': "ApplicationFault",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_insertion_order_serialize(
        self,
        update_insertion_order_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if update_insertion_order_request is not None:
            _body_params = update_insertion_order_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Authorization', 
            'UserName', 
            'CustomerAccountId', 
            'CustomerId', 
            'DeveloperToken', 
            'Password'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/CustomerBilling/v13/InsertionOrder',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class TrafficEstimate(BaseModel):
    """
    TrafficEstimate
    """ # noqa: E501
    impressions: Optional[StrictFloat] = Field(default=None, alias="Impressions")
    clicks: Optional[StrictFloat] = Field(default=None, alias="Clicks")
    total_cost: Optional[StrictFloat] = Field(default=None, alias="TotalCost")
    ctr: Optional[StrictFloat] = Field(default=None, alias="Ctr")
    average_cpc: Optional[StrictFloat] = Field(default=None, alias="AverageCpc")
    average_position: Optional[StrictFloat] = Field(default=None, alias="AveragePosition")
    __properties: ClassVar[List[str]] = ["Impressions", "Clicks", "TotalCost", "Ctr", "AverageCpc", "AveragePosition"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if impressions (nullable) is None
        # and model_fields_set contains the field
        if self.impressions is None and "impressions" in self.model_fields_set:
            _dict['Impressions'] = None

        # set to None if clicks (nullable) is None
        # and model_fields_set contains the field
        if self.clicks is None and "clicks" in self.model_fields_set:
            _dict['Clicks'] = None

        # set to None if total_cost (nullable) is None
        # and model_fields_set contains the field
        if self.total_cost is None and "total_cost" in self.model_fields_set:
            _dict['TotalCost'] = None

        # set to None if ctr (nullable) is None
        # and model_fields_set contains the field
        if self.ctr is None and "ctr" in self.model_fields_set:
            _dict['Ctr'] = None

        # set to None if average_cpc (nullable) is None
        # and model_fields_set contains the field
        if self.average_cpc is None and "average_cpc" in self.model_fields_set:
            _dict['AverageCpc'] = None

        # set to None if average_position (nullable) is None
        # and model_fields_set contains the field
        if self.average_position is None and "average_position" in self.model_fields_set:
            _dict['AveragePosition'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TrafficEstimate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Impressions": obj.get("Impressions") if obj.get("Impressions") is not None else None,
                        "Clicks": obj.get("Clicks") if obj.get("Clicks") is not None else None,
                        "TotalCost": obj.get("TotalCost") if obj.get("TotalCost") is not None else None,
                        "Ctr": obj.get("Ctr") if obj.get("Ctr") is not None else None,
                        "AverageCpc": obj.get("AverageCpc") if obj.get("AverageCpc") is not None else None,
                        "AveragePosition": obj.get("AveragePosition") if obj.get("AveragePosition") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.generic_entity_status import GenericEntityStatus
from typing_extensions import Self

class AssetGroupUrlTarget(BaseModel):
    """
    AssetGroupUrlTarget
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    target_condition1: Optional[StrictStr] = Field(default=None, alias="TargetCondition1")
    target_condition_operator1: Optional[StrictStr] = Field(default=None, alias="TargetConditionOperator1")
    target_value1: Optional[StrictStr] = Field(default=None, alias="TargetValue1")
    target_condition2: Optional[StrictStr] = Field(default=None, alias="TargetCondition2")
    target_condition_operator2: Optional[StrictStr] = Field(default=None, alias="TargetConditionOperator2")
    target_value2: Optional[StrictStr] = Field(default=None, alias="TargetValue2")
    target_condition3: Optional[StrictStr] = Field(default=None, alias="TargetCondition3")
    target_condition_operator3: Optional[StrictStr] = Field(default=None, alias="TargetConditionOperator3")
    target_value3: Optional[StrictStr] = Field(default=None, alias="TargetValue3")
    status: Optional[GenericEntityStatus] = Field(default=None, alias="Status")
    __properties: ClassVar[List[str]] = ["Id", "TargetCondition1", "TargetConditionOperator1", "TargetValue1", "TargetCondition2", "TargetConditionOperator2", "TargetValue2", "TargetCondition3", "TargetConditionOperator3", "TargetValue3", "Status"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if target_condition1 (nullable) is None
        # and model_fields_set contains the field
        if self.target_condition1 is None and "target_condition1" in self.model_fields_set:
            _dict['TargetCondition1'] = None

        # set to None if target_condition_operator1 (nullable) is None
        # and model_fields_set contains the field
        if self.target_condition_operator1 is None and "target_condition_operator1" in self.model_fields_set:
            _dict['TargetConditionOperator1'] = None

        # set to None if target_value1 (nullable) is None
        # and model_fields_set contains the field
        if self.target_value1 is None and "target_value1" in self.model_fields_set:
            _dict['TargetValue1'] = None

        # set to None if target_condition2 (nullable) is None
        # and model_fields_set contains the field
        if self.target_condition2 is None and "target_condition2" in self.model_fields_set:
            _dict['TargetCondition2'] = None

        # set to None if target_condition_operator2 (nullable) is None
        # and model_fields_set contains the field
        if self.target_condition_operator2 is None and "target_condition_operator2" in self.model_fields_set:
            _dict['TargetConditionOperator2'] = None

        # set to None if target_value2 (nullable) is None
        # and model_fields_set contains the field
        if self.target_value2 is None and "target_value2" in self.model_fields_set:
            _dict['TargetValue2'] = None

        # set to None if target_condition3 (nullable) is None
        # and model_fields_set contains the field
        if self.target_condition3 is None and "target_condition3" in self.model_fields_set:
            _dict['TargetCondition3'] = None

        # set to None if target_condition_operator3 (nullable) is None
        # and model_fields_set contains the field
        if self.target_condition_operator3 is None and "target_condition_operator3" in self.model_fields_set:
            _dict['TargetConditionOperator3'] = None

        # set to None if target_value3 (nullable) is None
        # and model_fields_set contains the field
        if self.target_value3 is None and "target_value3" in self.model_fields_set:
            _dict['TargetValue3'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AssetGroupUrlTarget from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "TargetCondition1": obj.get("TargetCondition1") if obj.get("TargetCondition1") is not None else None,
                        "TargetConditionOperator1": obj.get("TargetConditionOperator1") if obj.get("TargetConditionOperator1") is not None else None,
                        "TargetValue1": obj.get("TargetValue1") if obj.get("TargetValue1") is not None else None,
                        "TargetCondition2": obj.get("TargetCondition2") if obj.get("TargetCondition2") is not None else None,
                        "TargetConditionOperator2": obj.get("TargetConditionOperator2") if obj.get("TargetConditionOperator2") is not None else None,
                        "TargetValue2": obj.get("TargetValue2") if obj.get("TargetValue2") is not None else None,
                        "TargetCondition3": obj.get("TargetCondition3") if obj.get("TargetCondition3") is not None else None,
                        "TargetConditionOperator3": obj.get("TargetConditionOperator3") if obj.get("TargetConditionOperator3") is not None else None,
                        "TargetValue3": obj.get("TargetValue3") if obj.get("TargetValue3") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None
        })
        return _obj
